/*****************************************************************************/
/*                                                                           */
/*                                                                           */
/* Laser Worm                                                                */
/* ==========                                                                */
/*                                                                           */
/* Laser Worm is a simple but funny game! :-)                                */
/*                                                                           */
/* Set console mode or window mode and background colour in the file         */
/* "lworm.cfg".                                                              */
/*                                                                           */
/* Note, the graphics resolution for this program is 640 x 480 pixels        */
/* and can not be changed. For fullscreen mode, run the program in           */
/* console (DOS) mode under Windows.                                         */
/*                                                                           */
/*                                                                           */
/* Author etc                                                                */
/* ==========                                                                */
/*                                                                           */
/* Program version: lworm.c 1.21                                             */
/*                                                                           */
/* The program is free to copy and modify.                                   */
/*                                                                           */
/* Author: Stefan Spaennare, Lund, Sweden                                    */
/*                                                                           */
/* E-mail: stefan@spaennare.se                                               */
/*                                                                           */
/* Home page:    http://www.spaennare.se/index.html                          */
/* Program page: http://www.spaennare.se/ssprog2.html                        */
/*                                                                           */
/* BASIC II version:      Autumn 1985;  On FACIT DTC (about ABC 800)         */
/*                                      an old Swedish computer.             */
/* C version for DOS:     January 2000; Using Turbo C++ 2.0.                 */
/* C version for Windows: 2002-04-30;   Using Allegro game library and       */
/*                                      Microsoft Visual C++ 6.0.            */
/* Latest update:         2006-07-08                                         */
/*                                                                           */
/*                                                                           */
/* References                                                                */
/* ==========                                                                */
/*                                                                           */
/* 1. The program uses a random number generator ran2() from Numerical       */
/*    Recipes in C Second Edition.                                           */
/*                                                                           */
/*                                                                           */
/*****************************************************************************/

/*****************************************************************************/
/*                                                                           */
/* Notice:                                                                   */
/* =======                                                                   */
/*                                                                           */
/* I make no warranties that this program is (1) free of error, (2) con-     */
/* sistent with any standard merchantability, or (3) meeting the require-    */
/* ments of a particular application.  This software shall not, partly or    */
/* as a whole, participate in a process, whose outcome can result in injury  */
/* to a person or loss of property. It is solely designed for analytical     */
/* work.  Permission to use, copy, and distribute is hereby granted without  */
/* fee, providing that the header above including this notice appears in     */
/* all copies.                                                               */
/*                                                                           */
/*                                                         Stefan Spaennare  */
/*                                                                           */
/*****************************************************************************/


#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "allegro.h"


char cfg_name[21]="lworm.cfg";


#define IM1 2147483563
#define IM2 2147483399
#define AM (1.0/IM1)
#define IMM1 (IM1-1)
#define IA1 40014
#define IA2 40692
#define IQ1 53668
#define IQ2 52774
#define IR1 12211
#define IR2 3791
#define NTAB 32
#define NDIV (1+IMM1/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)
 

double pi=3.14159265358979323;

long x,y,xx1,yy1;
long cx,cy,d,n,ls;
long idum;
long candy_size,candy_counter;
long shot_counter,distance_counter;
long step_delay,laser_delay;
long border_col,worm_col,laser_col;
long candy_col,candy_old_col;
long text_col,background_col;

double skill;

int xv[20001];
int yv[20001];


int cr[21];
int cg[21];
int cb[21];

SAMPLE *sstart_sample,*laser_sample,*candy_sample,*hit_sample;


double ran2(idum)
long *idum;
{
   int j;
   long k;

   static long idum2=123456789;
   static long iy=0; 
   static long iv[NTAB];

   double temp;
 
   if (*idum <= 0) {
      if (-(*idum) < 1) {
         *idum=1;
      }
      else {
         *idum=-(*idum);
      } /* if */
      idum2=(*idum);
      for (j=NTAB+7; j>=0; j--) {
         k=(*idum)/IQ1;
	 *idum=IA1*(*idum-k*IQ1)-k*IR1;
	 if (*idum < 0) {
	    *idum += IM1;
	 } /* if */
	 if (j < NTAB) {
	    iv[j]=*idum;
	 } /* if */
      } /* for j */
      iy=iv[0];
   } /* if */
   k=(*idum)/IQ1;
   *idum=IA1*(*idum-k*IQ1)-k*IR1;
   if (*idum < 0) {
      *idum += IM1;
   } /* if */
   k=idum2/IQ2;
   idum2=IA2*(idum2-k*IQ2)-k*IR2;
   if (idum2 < 0) {
      idum2 += IM2;
   } /* if */
   j=iy/NDIV;
   iy=iv[j]-idum2;
   iv[j] = *idum;
   if (iy < 1) {
      iy += IMM1;
   } /* if */
   temp=AM*(double)(iy);
   if (temp > RNMX) {
      return(RNMX);
   }
   else {
      return(temp);
   } /* if */

} /* ran2 */ 


void start_sound()
{
   play_sample(sstart_sample,255,0,1000,1);
   rest(225);
   stop_sample(sstart_sample);
} /* start_sound */


void candy_sound()
{
   play_sample(candy_sample,255,0,1000,1);
   rest(50);
   stop_sample(candy_sample);   
} /* candy_sound */


void laser_sound()
{
   play_sample(laser_sample,255,0,1000,1);
   rest(32);
   stop_sample(laser_sample);
} /* laser_sound */


void hit_sound()
{
   play_sample(hit_sample,255,0,1000,1);
   rest(300);
   stop_sample(hit_sample);
} /* hit_sound */


void color_init()
{

   cr[0]=0;    cg[0]=0;    cb[0]=0;
   cr[1]=255;  cg[1]=255;  cb[1]=255;

   cr[2]=255;  cg[2]=0;    cb[2]=0;
   cr[3]=0;    cg[3]=255;  cb[3]=0;
   cr[4]=0;    cg[4]=0;    cb[4]=255;

   cr[5]=255;  cg[5]=0;    cb[5]=255;
   cr[6]=255;  cg[6]=255;  cb[6]=0;
   cr[7]=0;    cg[7]=255;  cb[7]=255;

   cr[8]=128;  cg[8]=0;    cb[8]=0;
   cr[9]=0;    cg[9]=128;  cb[9]=0;
   cr[10]=0;   cg[10]=0;   cb[10]=128;

   cr[11]=128; cg[11]=128; cb[11]=0;
   cr[12]=128; cg[12]=0;   cb[12]=128;
   cr[13]=0;   cg[13]=128; cb[13]=128;

   cr[14]=128; cg[14]=128; cb[14]=128;
   cr[15]=160; cg[15]=160; cb[15]=160;
   cr[16]=192; cg[16]=192; cb[16]=192;

} /* color_init */


void init()
{

   x=320;
   y=240;
   xx1=1;
   yy1=0;

   d=1;
   n=2;
   ls=4;

   candy_counter=-1;
   shot_counter=0;
   distance_counter=0;

   candy_size=24;

/*
   background_col=0;
*/
   laser_col=2;
   border_col=5;
   worm_col=2;
   candy_col=4;
   candy_old_col=3;
   text_col=2;

   step_delay=8; /* ms */
   laser_delay=16; /* ms */

   cx=(long)((637.0-(double)(candy_size))*ran2(&idum)+1.0);
   cy=(long)((464.0-(double)(candy_size))*ran2(&idum)+15.0);

} /* init */


void empty()
{
  int j,col;

   col=makecol(cr[background_col],cg[background_col],cb[background_col]);

   for (j=0; j<=639; j++) {
      line(screen,j,0,j,479,col);
   } /* for j */

} /* empty */


void text_empty()
{
  int j,col;

   col=makecol(cr[background_col],cg[background_col],cb[background_col]);

   for (j=1; j<=14; j++) {
      line(screen,1,j,638,j,col);
   } /* for j */

} /* text_empty */


void hit_empty()
{
  int j,col;

   col=makecol(cr[background_col],cg[background_col],cb[background_col]);

   for (j=180; j<=269; j++) {
      line(screen,210,j,419,j,col);
   } /* for j */

} /* hit_empty */


void hit_border()
{
  int col;

   col=makecol(cr[border_col],cg[border_col],cb[border_col]);

   line(screen,210,180,419,180,col);
   line(screen,419,180,419,269,col);
   line(screen,419,269,210,269,col);
   line(screen,210,269,210,180,col);

} /* hit_border */


void start_border()
{
  int col;

   col=makecol(cr[border_col],cg[border_col],cb[border_col]);

   line(screen,210,190,419,190,col);
   line(screen,419,190,419,259,col);
   line(screen,419,259,210,259,col);
   line(screen,210,259,210,190,col);

   line(screen,0,0,639,0,col);
   line(screen,639,0,639,479,col);
   line(screen,639,479,0,479,col);
   line(screen,0,479,0,0,col);

} /* start_border */


void print_text()
{
   int col;

   text_empty();

   text_mode(-1);

   col=makecol(cr[text_col],cg[text_col],cb[text_col]);

   textprintf(screen,font,20,6,col,"Candy: %8ld",candy_counter);
   textprintf(screen,font,220,6,col,"Shots: %8ld",shot_counter);
   textprintf(screen,font,420,6,col,"Distance: %8ld",distance_counter);

} /* print_text */



void border()
{

   int col;

   print_text();

   col=makecol(cr[border_col],cg[border_col],cb[border_col]);

   line(screen,0,0,639,0,col);
   line(screen,639,0,639,479,col);
   line(screen,639,479,0,479,col);
   line(screen,0,479,0,0,col);

   line(screen,0,15,639,15,col);

} /* border */


void start()
{
   int col;

   start_border();

   text_mode(-1);
   
   col=makecol(cr[text_col],cg[text_col],cb[text_col]);

   textprintf(screen,font,220,200,col,"Laser Worm version 1.20");
   textprintf(screen,font,220,220,col,"Steer with s and l");
   textprintf(screen,font,220,240,col,"Laser with space");

   start_sound();

} /* start */


void candy()
{
   int j;

   int col;

   col=makecol(cr[candy_old_col],cg[candy_old_col],cb[candy_old_col]);

   for (j=cx; j<=(cx+candy_size); j++) {
      line(screen,j,cy,j,cy+candy_size,col);
   } /* for j */

   cx=(long)((637.0-(double)(candy_size))*ran2(&idum)+1.0);
   cy=(long)((464.0-(double)(candy_size))*ran2(&idum)+15.0);

   col=makecol(cr[candy_col],cg[candy_col],cb[candy_col]);

   for (j=cx; j<=(cx+candy_size); j++) {
      line(screen,j,cy,j,cy+candy_size,col);
   } /* for j */

   candy_counter=candy_counter+1;

   print_text();

   candy_sound();

} /* candy */


void laser()
{

   int i,lcol,ccol,col0;

   shot_counter=shot_counter+1;

   print_text();

   col0=makecol(cr[background_col],cg[background_col],cb[background_col]);

   ccol=makecol(cr[candy_col],cg[candy_col],cb[candy_col]);

   lcol=makecol(cr[laser_col],cg[laser_col],cb[laser_col]);

   if (xx1 >= 1) {
      for (i=(x+1); i<=(638-ls); i=i+ls) {
	 if (getpixel(screen,i,y) == ccol) {
	    candy();
	 } /* if */
	 putpixel(screen,i,y,lcol);
      } /* for i */
      rest(laser_delay);
      i=i-ls;
      line(screen,i,y,x+1,y,col0);
   } /* if */

   if (xx1 <= -1) {
      for (i=(x-1); i>=(1+ls); i=i-ls) {
	 if (getpixel(screen,i,y) == ccol) {
	    candy();
	 } /* if */
	 putpixel(screen,i,y,lcol);
      } /* for i */
      rest(laser_delay);
      i=i+ls;
      line(screen,i,y,x-1,y,col0);
   } /* if */

   if (yy1 >= 1) {
      for (i=(y+1); i<=(478-ls); i=i+ls) {
	 if (getpixel(screen,x,i) == ccol) {
	    candy();
	 } /* if */
	 putpixel(screen,x,i,lcol);
      } /* for i */
      rest(laser_delay);
      i=i-ls;
      line(screen,x,i,x,y+1,col0);
   } /* if */

   if (yy1 <= -1) {
      for (i=(y-1); i >=(14+ls); i=i-ls) {
	 if (getpixel(screen,x,i) == ccol) {
	    candy();
	 } /* if */
	 putpixel(screen,x,i,lcol);
      } /* for */
      rest(laser_delay);
      i=i+ls;
      line(screen,x,i,x,y-1,col0);
   } /* if */

   laser_sound();

} /* laser */


void hit()
{
     int col;

     double t;

     print_text();

     if ((distance_counter == 0) || (shot_counter == 0)) {
	skill=0.0;
     }
     else {
	t=(double)(candy_counter);
	skill=15.0*t*t*t/
	((double)(distance_counter)*(double)(shot_counter));
     } /* if */

     hit_empty();
     hit_border();

     col=makecol(cr[text_col],cg[text_col],cb[text_col]);

     textprintf(screen,font,220,190,col,"Candy points:    %6ld",candy_counter);
     textprintf(screen,font,220,210,col,"Distance points: %6ld",distance_counter);
     textprintf(screen,font,220,230,col,"Running skill:   %6.2lf",skill);

     hit_sound();

} /* hit */


int main()
{

   FILE *cfgfile;

   long flag,seed;

   int col,col0,ccol,j;

   int xsize,ysize;

   int wintype,bgcol;

   char cc;

   double ttt1,ttt2;
 
   time_t tt;
 
   tt=time(0); 
   ttt1=(double)(tt)+0.5; 
   ttt2=fmod(ttt1,2147483647.0); 
   seed=(long)(ttt2); 
   idum=-seed-1;


   xsize=640;
   ysize=480;


   if ((cfgfile = fopen(cfg_name,"r")) == NULL) {
      printf("File %s not found.\n",cfg_name);
      exit(0);
   } /* if */


   fscanf(cfgfile,"%d",&wintype);
   fscanf(cfgfile,"%d",&bgcol);


   if ((wintype < 0) || (wintype > 1)) {
      printf("wintype must be 0 or 1.\n");
      exit(0);
   } /* if */

   if ((bgcol < 0) || (bgcol > 1)) {
      printf("bgcol must be 0 or 1.\n");
      exit(0);
   } /* if */


   color_init();

   allegro_init();

   install_keyboard();

   set_color_depth(32);


   if (wintype == 0) {
      if (set_gfx_mode(GFX_AUTODETECT,xsize,ysize,0,0) < 0) {
         if (set_gfx_mode(GFX_AUTODETECT_WINDOWED,xsize,ysize,0,0) < 0) {
            allegro_exit();
            printf("Graphics error or wrong resolution.\n");
            exit(0);
         } /* if */
      } /* if */
   } /* if */

   if (wintype == 1) {
      if (set_gfx_mode(GFX_AUTODETECT_WINDOWED,xsize,ysize,0,0) < 0) {
         if (set_gfx_mode(GFX_AUTODETECT,xsize,ysize,0,0) < 0) {
            allegro_exit();
            printf("Graphics error or wrong resolution.\n");
            exit(0);
         } /* if */
      } /* if */
   } /* if */


   set_palette(desktop_palette);


   if (install_sound(DIGI_AUTODETECT,MIDI_AUTODETECT,NULL) != 0) {
      install_sound(DIGI_NONE,MIDI_NONE,NULL);
   } /* if */
       
   sstart_sample=load_sample("start.wav");
   laser_sample=load_sample("laser.wav");
   candy_sample=load_sample("candy.wav");
   hit_sample=load_sample("hit.wav");


   cc='y';

   flag=0;

while (cc != 'n') {

   background_col=bgcol;

   init();
   empty();
   start();

   for (j=1; j<=20000; j++) {
      xv[j]=0;
      yv[j]=0;
   } /* for j */

   while (!keypressed()) {
   } /* while */

   cc=(readkey() & 0xff);

   empty();
   border();
   candy();

   d=1;
   xv[d]=x;
   yv[d]=y;

   while (!keypressed()) {
   } /* while */

   cc=(readkey() & 0xff);

   while (flag == 0) {

      if (keypressed()) {

	 cc=(readkey() & 0xff);

	 if (cc == ' ') {
	    laser();
	 }
	 else if (cc == 's') {
	    if (yy1 == 0) {
	       yy1=-xx1;
	       xx1=0;
	    }
	    else {
	       xx1=yy1;
	       yy1=0;
	    } /* if */
	 }
	 else if (cc == 'l') {
	    if (yy1 == 0) {
	       yy1=xx1;
	       xx1=0;
	    }
	    else {
	       xx1=-yy1;
	       yy1=0;
	    } /* if */
	 } /* if */

      } /* if */

	 x=x+xx1;
	 y=y+yy1;

         if ((xv[d] != 0) && (yv[d] != 0)) {
            col0=makecol(cr[background_col],cg[background_col],cb[background_col]);
            putpixel(screen,xv[d],yv[d],col0);
         } /* if */

	 xv[d]=x;
	 yv[d]=y;

	 d=d+1;
	 if (d == n) {
	    d=1;
	    n=n+1+n/10;
	    print_text();
	 } /* if */

	 if ((d % 10) == 0) {
	    if ((ran2(&idum)) < 0.002) {
	       candy_counter=candy_counter-1;
	       candy();
	    } /* if */
	 } /* if */

	 col=getpixel(screen,x,y);

         col0=makecol(cr[background_col],cg[background_col],cb[background_col]);

	 if (col != col0) {
	    hit();
	    flag=1;
	 }
         else {
            ccol=makecol(cr[worm_col],cg[worm_col],cb[worm_col]);
	    putpixel(screen,x,y,ccol);
	    distance_counter=distance_counter+1;
            rest(step_delay);
         } /* if */

     } /* while */


     ccol=makecol(cr[text_col],cg[text_col],cb[text_col]);

     textprintf(screen,font,220,250,ccol,"New game (y/n)?");

     do {
        if (keypressed()) {
           cc=(readkey() & 0xff);
        } /* if */
	flag=0;
     } while ((cc != 'y') && (cc != 'n'));

   } /* while */

   destroy_sample(sstart_sample);
   destroy_sample(laser_sample);
   destroy_sample(candy_sample);
   destroy_sample(hit_sample);

   remove_sound();

   allegro_exit();

   fclose(cfgfile);

   return(0);

} /* End  */

END_OF_MAIN();

