/*****************************************************************************/
/*                                                                           */
/*                                                                           */
/* Image transformation                                                      */
/* ====================                                                      */
/*                                                                           */
/* This program transforms a simple image square.                            */
/*                                                                           */
/* Set graphics resolution, console mode or window mode, background colour   */  
/* etc and the transformation parameters in the file "transf.cfg".           */
/*                                                                           */
/*                                                                           */
/* Author etc                                                                */
/* ==========                                                                */
/*                                                                           */
/* Program version: transf.c 1.20                                            */
/*                                                                           */
/* The program is free to copy and modify.                                   */
/*                                                                           */
/* Author: Stefan Spaennare, Lund, Sweden                                    */
/*                                                                           */
/* E-mail: stefan@spaennare.se                                               */
/*                                                                           */
/* Home page:    http://www.spaennare.se/index.html                          */
/* Program page: http://www.spaennare.se/ssprog2.html                        */
/*                                                                           */
/* C version for Windows: 2006-07-13;   Using Allegro game library and       */
/*                                      Microsoft Visual C++ 6.0.            */
/* Latest update:         2006-07-14                                         */
/*                                                                           */
/*                                                                           */
/* References                                                                */
/* ==========                                                                */
/*                                                                           */
/* 1. The program uses a random number generator ran2() from Numerical       */
/*    Recipes in C Second Edition.                                           */
/*                                                                           */
/*                                                                           */
/*****************************************************************************/

/*****************************************************************************/
/*                                                                           */
/* Notice:                                                                   */
/* =======                                                                   */
/*                                                                           */
/* I make no warranties that this program is (1) free of error, (2) con-     */
/* sistent with any standard merchantability, or (3) meeting the require-    */
/* ments of a particular application.  This software shall not, partly or    */
/* as a whole, participate in a process, whose outcome can result in injury  */
/* to a person or loss of property. It is solely designed for analytical     */
/* work.  Permission to use, copy, and distribute is hereby granted without  */
/* fee, providing that the header above including this notice appears in     */
/* all copies.                                                               */
/*                                                                           */
/*                                                         Stefan Spaennare  */
/*                                                                           */
/*****************************************************************************/


#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include "allegro.h"

double pi=3.14159265358979323;

double timetic=(double)(CLOCKS_PER_SEC);

char cfg_name[21]="transf.cfg";

char image_name[21]="transf.bmp";


#define IM1 2147483563
#define IM2 2147483399
#define AM (1.0/IM1)
#define IMM1 (IM1-1)
#define IA1 40014
#define IA2 40692
#define IQ1 53668
#define IQ2 52774
#define IR1 12211
#define IR2 3791
#define NTAB 32
#define NDIV (1+IMM1/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)

double ran2(idum)
long *idum;
{
   int j;
   long k;
   static long idum2=123456789;
   static long iy=0; 
   static long iv[NTAB];
   double temp;

   if (*idum <= 0) {
      if (-(*idum) < 1) {
         *idum=1;
      }
      else {
         *idum=-(*idum);
      } /* if */
      idum2=(*idum);
      for (j=NTAB+7; j>=0; j--) {
         k=(*idum)/IQ1;
	 *idum=IA1*(*idum-k*IQ1)-k*IR1;
	 if (*idum < 0) {
	    *idum += IM1;
	 } /* if */
	 if (j < NTAB) {
	    iv[j]=*idum;
	 } /* if */
      } /* for j */
      iy=iv[0];
   } /* if */
   k=(*idum)/IQ1;
   *idum=IA1*(*idum-k*IQ1)-k*IR1;
   if (*idum < 0) {
      *idum += IM1;
   } /* if */
   k=idum2/IQ2;
   idum2=IA2*(idum2-k*IQ2)-k*IR2;
   if (idum2 < 0) {
      idum2 += IM2;
   } /* if */
   j=iy/NDIV;
   iy=iv[j]-idum2;
   iv[j] = *idum;
   if (iy < 1) {
      iy += IMM1;
   } /* if */
   temp=AM*(double)(iy);
   if (temp > RNMX) {
      return(RNMX);
   }
   else {
      return(temp);
   } /* if */

} /* ran2 */


int main(argc,argv)
int argc;
char *argv[];
{

   FILE *cfgfile;

   BITMAP *bmp;
   PALETTE pal;

   int xsize,ysize,col,c0;
   int wintype,imsave,delay0,xsize22;
   int col1,col2,col3,col4,bgcol,bordercol,textcol;
   int x,y,i,j,k,x0,y0,x1,y1,size,size1,fac,fac2,fac3;
   int auto1,xsize2,xsize4,xoff,flag,recflag,temp;
   int c11,c12,c13,c21,c22,c23;

   double ttt1,ttt2,sum,totsum;

   char ch;

   long idum,seed;

   int cr[21];
   int cg[21];
   int cb[21];

   time_t tt;

   clock_t time1,timediff1;


   if ((cfgfile = fopen(cfg_name,"r")) == NULL) {
      printf("File %s not found.\n",cfg_name);
      exit(0);
   } /* if */


   fscanf(cfgfile,"%d %d",&xsize,&ysize);
   fscanf(cfgfile,"%d",&wintype);
   fscanf(cfgfile,"%d",&imsave);

   fscanf(cfgfile,"%d",&size);

   fscanf(cfgfile,"%d",&delay0);

   fscanf(cfgfile,"%d",&auto1);

   fscanf(cfgfile,"%d",&recflag);

   fscanf(cfgfile,"%d",&bgcol);
   fscanf(cfgfile,"%d",&bordercol);
   fscanf(cfgfile,"%d",&textcol);
   fscanf(cfgfile,"%d",&col1);
   fscanf(cfgfile,"%d",&col2);
   fscanf(cfgfile,"%d",&col3);
   fscanf(cfgfile,"%d",&col4);

   fscanf(cfgfile,"%d %d %d",&c11,&c12,&c13);
   fscanf(cfgfile,"%d %d %d",&c21,&c22,&c23);


   if ((wintype < 0) || (wintype > 1)) {
      printf("wintype must be 0 or 1.\n");
      exit(0);
   } /* if */

   if ((imsave < 0) || (imsave > 1)) {
      printf("imsave must be 0 or 1.\n");
      exit(0);
   } /* if */

   if ((auto1 < 0) || (auto1 > 1)) {
      printf("auto1 must be 0 or 1.\n");
      exit(0);
   } /* if */

   if ((recflag < 0) || (recflag > 1)) {
      printf("recflag must be 0 or 1.\n");
      exit(0);
   } /* if */


   allegro_init();

   install_keyboard();

   set_color_depth(32);


   if (wintype == 0) {
      if (set_gfx_mode(GFX_AUTODETECT,xsize,ysize,0,0) < 0) {
         if (set_gfx_mode(GFX_AUTODETECT_WINDOWED,xsize,ysize,0,0) < 0) {
            allegro_exit();
            printf("Graphics error or wrong resolution.\n");
            printf("Try to set resolution to 640 480.\n");
            exit(0);
         } /* if */
      } /* if */
   } /* if */

   if (wintype == 1) {
      if (set_gfx_mode(GFX_AUTODETECT_WINDOWED,xsize,ysize,0,0) < 0) {
         if (set_gfx_mode(GFX_AUTODETECT,xsize,ysize,0,0) < 0) {
            allegro_exit();
            printf("Graphics error or wrong resolution.\n");
            printf("Try to set resolution to 640 480.\n");
            exit(0);
         } /* if */
      } /* if */
   } /* if */


   set_palette(desktop_palette);


   cr[0]=0;    cg[0]=0;    cb[0]=0;
   cr[1]=255;  cg[1]=255;  cb[1]=255;

   cr[2]=255;  cg[2]=0;    cb[2]=0;
   cr[3]=0;    cg[3]=255;  cb[3]=0;
   cr[4]=0;    cg[4]=0;    cb[4]=255;

   cr[5]=255;  cg[5]=0;    cb[5]=255;
   cr[6]=255;  cg[6]=255;  cb[6]=0;
   cr[7]=0;    cg[7]=255;  cb[7]=255;

   cr[8]=128;  cg[8]=0;    cb[8]=0;
   cr[9]=0;    cg[9]=128;  cb[9]=0;
   cr[10]=0;   cg[10]=0;   cb[10]=128;

   cr[11]=128; cg[11]=128; cb[11]=0;
   cr[12]=128; cg[12]=0;   cb[12]=128;
   cr[13]=0;   cg[13]=128; cb[13]=128;

   cr[14]=128; cg[14]=128; cb[14]=128;
   cr[15]=160; cg[15]=160; cb[15]=160;
   cr[16]=192; cg[16]=192; cb[16]=192;


   tt=time(0);
   ttt1=(double)(tt)+0.5;
   ttt2=fmod(ttt1,2147483647.0);
   seed=(long)(ttt2);
   idum=-seed-1;


   xsize2=xsize/2;
   xsize22=xsize2-2;
   xsize4=xsize/4;

   if (size > xsize22) {
      size=xsize22;
   } /* if */

   c0=bgcol;
   col=makecol(cr[c0],cg[c0],cb[c0]);

   for (i=0; i<ysize; i++) {
      line(screen,0,i,xsize-1,i,col);
   } /* for i */ 

   c0=bordercol;
   col=makecol(cr[c0],cg[c0],cb[c0]);
   line(screen,0,0,xsize-1,0,col);
   line(screen,xsize-1,0,xsize-1,ysize-1,col);
   line(screen,xsize-1,ysize-1,0,ysize-1,col);
   line(screen,0,ysize-1,0,0,col);

   line(screen,xsize2,0,xsize2,ysize-1,col);
   line(screen,0,xsize2,xsize-1,xsize2,col);
   line(screen,0,xsize2+16,xsize-1,xsize2+16,col);

   text_mode(-1);

   c0=textcol;
   col=makecol(cr[c0],cg[c0],cb[c0]);
   textprintf(screen,font,xsize/12,xsize2+xsize/12,col,   "Transf. square size: %4d",size);
   textprintf(screen,font,xsize/12,xsize2+xsize/12+30,col,"Auto run on/off:  press a");
   textprintf(screen,font,xsize/12,xsize2+xsize/12+50,col,"Quit:             press q");

   size1=size-1;

   xoff=(xsize2-size)/2;


   fac=size/8;
   fac2=size/4;
   fac3=size/6;

   c0=col1;
   col=makecol(cr[c0],cg[c0],cb[c0]);

   for (i=0; i<=size1; i++) {
      for (j=0; j<=size1; j++) {
	 putpixel(screen,i+xoff,j+xoff,col);
      } /* for j */
   } /* for i */

   c0=col2;
   col=makecol(cr[c0],cg[c0],cb[c0]);

   for (i=fac; i<=(size1-fac); i++) {
      for (j=fac; j<=(size1-fac); j++) {
	 putpixel(screen,i+xoff,j+xoff,col);
      } /* for j */
   } /* for i */

   c0=col3;
   col=makecol(cr[c0],cg[c0],cb[c0]);

   for (i=fac2; i<=(size1-fac2); i++) {
      for (j=fac2; j<=(size1-fac2); j++) {
	 putpixel(screen,i+xoff,j+xoff,col);
      } /* for j */
   } /* for i */

   c0=col4;
   col=makecol(cr[c0],cg[c0],cb[c0]);

   line(screen,fac3+xoff,size1-fac3+xoff,size1-fac3+xoff,size1-fac3+xoff,col);


   x0=xoff;
   y0=xoff;
   x1=xsize2+xoff;
   y1=xoff;

   c0=bgcol;
   col=makecol(cr[c0],cg[c0],cb[c0]);
   rectfill(screen,x0-xoff+xsize4-50,xsize2+1,x0-xoff+xsize4+20,xsize2+15,col);

   k=0;

   text_mode(-1);

   c0=textcol;
   col=makecol(cr[c0],cg[c0],cb[c0]);
   textprintf(screen,font,x0-xoff+xsize4-30,xsize2+6,col,"%6d",k);


   ch='#';

   flag=0;

   delay0=100*delay0;

   totsum=0.0;


do {


   do {


      if (recflag == 1) {
         c0=bgcol;
         col=makecol(cr[c0],cg[c0],cb[c0]);
         rectfill(screen,x1,y1,x1+size,y1+size,col);
      } /* if */


      for (i=0; i<=size1; i++) {
	 for (j=0; j<=size1; j++) {
	    col=getpixel(screen,i+x0,j+y0);
	    x=(c11*i+c12*j+c13) % size;
	    y=(c21*i+c22*j+c23) % size;
	    putpixel(screen,x+x1,y+y1,col);
	 } /* for j */
      } /* for i */

      k++;

      c0=bgcol;
      col=makecol(cr[c0],cg[c0],cb[c0]);
      rectfill(screen,x1-xoff+xsize4-50,xsize2+1,x1-xoff+xsize4+20,xsize2+15,col);

      text_mode(-1);

      c0=textcol;
      col=makecol(cr[c0],cg[c0],cb[c0]);
      textprintf(screen,font,x1-xoff+xsize4-30,xsize2+6,col,"%6d",k);

      temp=x0;
      x0=x1;
      x1=temp;

      sum=0.0;

      for (j=1; j<=delay0; j++) {
         sum=sum+1.0/(double)(j);
      } /* for j */

      totsum=totsum+sum;


   } while ((!keypressed()) && (auto1==1));


      while (!keypressed()) {
      } /* while */

      if (keypressed()) {

	 ch=(readkey() & 0xff);

	 if (ch == 'a') {
             if (auto1==0) {
	        auto1=1;
             }
             else {
	        auto1=0;
             } /* if */
	 } /* if */

      } /* if */


} while (ch != 'q');


   if (imsave == 1) {
      get_palette(pal);
      bmp=create_sub_bitmap(screen,0,0,xsize,ysize);
      save_bitmap(image_name,bmp,pal);
      destroy_bitmap(bmp);
   } /* if */

   allegro_exit();

   fclose(cfgfile);

   printf("dummy = %12.6f\n",totsum);

   return(0);

} /* End  */

END_OF_MAIN();
